/**
 * @version 3.2 2004-06-1
 * @author Gonzalo Ojeda 9821018-0
 */

import java.awt.*;
import javax.swing.*;
import java.awt.event.*;
import javax.swing.JFileChooser.*;
import java.io.*;
import javax.swing.filechooser.FileFilter;
import java.util.*;

/**
 * La clase <b>EditorFrame</b> corresponde a la encargada de crear el frame 
 * principal del programa, el cual como se explicar en su momento contiene 2 
 * JPanel en si, uno para contener un panel de botones, y otro para la zona de
 * trabajo.
 */
public class EditorFrame extends JFrame 
{
 /**
  * Constructor de la clase <b>EditorFrame</b>, la cual setea el tamao de la
  * ventana de programa, as mismo crea los 2 JPanel, uno llamado <i>
  * buttonPanel</i> el cual es un JPanel y como su nombre lo indica contendr
  * los 7 botones que se definen, y otro llamado <i>fPanel</i> que es un objeto
  * de la clase <b>FigPanel</b> y corresponde a la zona donde se dibujaran las 
  * figuras.
  */  
   public EditorFrame()
   {
      // Botones para la creacin de las Figuras.  
      JButton rectButton = new JButton ("Rectangulo");
      JButton elipButton = new JButton ("Elipse");
      JButton linButton = new JButton ("Linea");
   
      // Botones para las operaciones sobre las Figuras.
      JButton rotButton = new JButton ("Rotar");
      JButton escButton = new JButton ("Escalar");
      JButton trasButton = new JButton ("Trasladar");
   
      // Boton que limpia la zona de dibujo.
      JButton clearButton = new JButton ("Limpiar");

      // Definicin de el tamao de la ventana creada.
      final int DEFAULT_WIDTH = 600;
      final int DEFAULT_HEIGHT = 400;
      setSize(DEFAULT_WIDTH, DEFAULT_HEIGHT);

      // Crear un Action Listener para el boton que limpia la zona de dibujo, y
      // asigna este ActionListener al botn para limpiar la zona de dibujo.
      ActionListener listener = new CleanListener();
      clearButton.addActionListener (listener);

      // Panel donde se ubicarn los Botones.
      JPanel buttonPanel = new JPanel();

      // habilita para poder usar sub secciones del JPanel, para distribuir los
      // botones.
      buttonPanel.setLayout(new BorderLayout());
   
      // Un panel superior que ira a la izquierda, donde iran los botones de
      // formas tambin se lo habilita para poder usar subsecciones de l.
      JPanel northPanel = new JPanel ();
      northPanel.setLayout(new BorderLayout());
   
      // se agregan los botones de figura a este JPanel, en distintas zonas.
      northPanel.add(rectButton, BorderLayout.NORTH);
      northPanel.add(elipButton, BorderLayout.CENTER);
      northPanel.add(linButton, BorderLayout.SOUTH);

      // Un panel inferior donde iran las opciones
      JPanel southPanel = new JPanel();
      southPanel.setLayout(new BorderLayout());
   
      // se agregan los botones con las opciones
      southPanel.add(rotButton, BorderLayout.NORTH);
      southPanel.add(escButton, BorderLayout.CENTER);
      southPanel.add(trasButton, BorderLayout.SOUTH);

      // se agregan al panel de botones, los dos paneles antes creados con los
      // botones para figuras y opciones respectivamente, adems de el botn 
      // para limpiar.
      buttonPanel.add(northPanel, BorderLayout.NORTH);
      buttonPanel.add(clearButton, BorderLayout.CENTER);
      buttonPanel.add(southPanel, BorderLayout.SOUTH);
   
      // Crear Panel donde se dibujaran las figuras, se le pasan los botones
      // pues los ActionListener se crearn en este panel FigPanel.
      fPanel = new FigPanel(rectButton, elipButton, linButton, rotButton, 
         escButton, trasButton);

      // crea un menu, y agrega la opcion File a este
      JMenuBar menuBar = new JMenuBar();
      setJMenuBar(menuBar);
      JMenu fileMenu = new JMenu("File");
      menuBar.add(fileMenu);
      // agrega la opcin Open a File y define un acelerador (ctrl + O)
      JMenuItem openItem = new JMenuItem("Open");
      openItem.setAccelerator(KeyStroke.getKeyStroke(
      KeyEvent.VK_O, ActionEvent.CTRL_MASK));
      openItem.addActionListener(new ActionListener() 
      { 
        // metodo que indica la accion asociada a "Open"
        public void actionPerformed(ActionEvent ae) 
        {
          //crea un FileChooser y un filtro para extensiones
          JFileChooser chooser = new JFileChooser(".");
          ExtensionFileFilter filter = new ExtensionFileFilter();
          filter.addExtension(".eda");       // solo abrir archivos ".eda"
          filter.setDescription("Archivos EDA (Figuras)"); 
          chooser.setAcceptAllFileFilterUsed(false); //no mostrar todos
          chooser.setFileFilter(filter);     // asignar filtro al FileChooser
          // mostrar FileChooser
          int option = chooser.showOpenDialog(EditorFrame.this);
          if (option == JFileChooser.APPROVE_OPTION) //si se escogi un archivo
          {
            File sf = chooser.getSelectedFile();
            fileName = sf.getName();      // se obtiene el nombre del archivo
            // leer el archivo y capturar las excepciones correspondientes
            try 
            {
               fPanel.read(fileName);     // llama a la funcin para leer
            } 
            catch (IOException e) 
              {
                System.err.println("Caught IOException: " + e.getMessage());
              } 
              catch (ClassNotFoundException e) 
                {
                   System.err.println("Caught ClassNotFoundException: " 
                      + e.getMessage());
                }

          } 
        } }); 
      fileMenu.add(openItem);       // agrega el "Open" a "File"
      // agregar opcion save y definir acelerador (ctrl + S)
      JMenuItem saveItem = new JMenuItem("Save");
      saveItem.setAccelerator(KeyStroke.getKeyStroke(
      KeyEvent.VK_S, ActionEvent.CTRL_MASK));
      saveItem.addActionListener(new ActionListener() 
      { 
        // mtodo asociado a la accin "Save"
        public void actionPerformed(ActionEvent ae) 
        {
          //crea un FileChooser y un filtro para extensiones
          JFileChooser chooser = new JFileChooser(".");
          ExtensionFileFilter filter = new ExtensionFileFilter();
          filter.addExtension(".eda");       // solo abrir archivos ".eda"
          filter.setDescription("Archivos EDA (Figuras)"); 
          chooser.setAcceptAllFileFilterUsed(false); //no mostrar todos
          chooser.setFileFilter(filter);     // asignar filtro al FileChooser
          // mostrar FileChooser
          int option = chooser.showSaveDialog(EditorFrame.this);
          if (option == JFileChooser.APPROVE_OPTION) //si se escogi un archivo
          {
            File sf = chooser.getSelectedFile();
            fileName = sf.getName();   // se obtiene el nombre del archivo
            // leer el archivo y capturar las excepciones correspondientes
            try 
            {
               fPanel.save(fileName);  //llama al mtodo que guarda en archivo
            } 
            catch (IOException exception)
            {
               exception.printStackTrace();
            }
          }
         }
      }); 
      fileMenu.add(saveItem);    // agrega el "Save" a "File"
      
      // crea y agrega About y Exit al menu dentro de File
            JMenuItem aboutItem = new JMenuItem("About");
            aboutItem.addActionListener(new
            ActionListener()
            {
                  // la opcion About genera un popup con informacion
         public void actionPerformed(ActionEvent event)
                  {
                     if (dialog == null) // primera vez
                        dialog = new AboutDialog(EditorFrame.this);   
                     dialog.show(); // pop up dialog
                  }
          });
            fileMenu.add(aboutItem);

      // La opcion Exit sale del programa
      JMenuItem exitItem = new JMenuItem("Exit");
            exitItem.addActionListener(new
            ActionListener()
            {
                  public void actionPerformed(ActionEvent event)
                  {
                        System.exit(0);
                  }
            });
            fileMenu.add(exitItem);

      // Agregar el panel de botones a la izquierda y el panel para las figuras
      // al centro del panel central.
      Container contentPane = getContentPane();
      contentPane.add(buttonPanel, BorderLayout.WEST);
      contentPane.add(fPanel, BorderLayout.CENTER);
   }
   /**
    * Clase interna para responder a los click en el boton que limpia la zona de
    * dibujo.
    * Al realizarse un click en este botn, se llama a un mtodo definido en la
    * clase <b>FigPanel</b>, el cual borra la zona de dibujo.
    */
   private class CleanListener implements ActionListener
   {
      /**
       * Mtodo que responde a la accin de presionar el botn de limpiado,
       * invoca al mtodo clearShapes() definido en la clase <b>FigPanel</b>.
       */   
      public void actionPerformed(ActionEvent e)
      {
         fPanel.clearShapes();
      }
   }
   
   // Objeto perteneciente a la clase <b>FigPanel</b> se usa para crear la zona
   // de dibujo, y para invocar a la funcion que limpia esta misma zona.
   private FigPanel fPanel;
   // Objeto para mostrar el popup al seleccionar la opcion About
   private AboutDialog dialog; 
   private String fileName;
}

/**
   Este file filter acepta todos los archivos que contengan un set de 
   extensiones dadas. Sacada de FileChooserTest en el Capitulo 9, texto gua
*/
class ExtensionFileFilter extends FileFilter
{
   /**
      Agrega una extensin que este file filter reconoce.
      @param extension una extensin de archivo (ej ".eda" or "eda")
   */
   public void addExtension(String extension)
   {
      if (!extension.startsWith("."))
         extension = "." + extension;
      extensions.add(extension.toLowerCase());     
   }

   /**
      Pone una descripcin para cada una de las extensiones que el file filter
      reconoce.
      @param aDescription descripcin asociada a una extensin.
   */
   public void setDescription(String aDescription)
   {
      description = aDescription;
   }

   /**
      Retorna la descripcin asociada a las extensiones que el file filter
      reconoce.
      @return una descripcin para un tipo de archivos dado
   */
   public String getDescription()
   {
      return description; 
   }

   public boolean accept(File f)
   {
      if (f.isDirectory()) return true;
      String name = f.getName().toLowerCase();

      // revisa si el archivo termina con alguna de las extensiones aceptadas
      for (int i = 0; i < extensions.size(); i++)
         if (name.endsWith((String)extensions.get(i))) 
            return true;
      return false;
   }
   
   private String description = "";
   private ArrayList extensions = new ArrayList();
}
