/**
 * @version 1.0 2004-06-1
 * @author Gonzalo Ojeda 9821018-0
 */

import java.awt.geom.*;
import java.io.*;

/**
 * Clase utilizada para manejar las figuras, y poder guardarlas como objeto
 * (serializar), es una clase abstracta que ser extendida por las clases
 * correspondientes a cada figura.
 */
 public abstract class Formas implements Serializable
{
   /**
    * Constructor de la clase, define los puntos que corresponden al inicio
    * y final de la figura.
    * @param punto1 corresponde al punto inicial de la figura
    * @param punto2 corresponde al punto final de la figura
    */
   public Formas (Point2D punto1, Point2D punto2)
   {
      this.punto1 = punto1;
      this.punto2 = punto2;
   }

   /**
    * Mtodo abstracta que se usa para devolver un Shape (aunque en este caso
    * se usa un Object) ser implementada en cada una de las clases derivadas
    * @return Un Shape (en este caso un Object)
    */
   public abstract Object setForma();

   /**
    * Mtodo que setea el segundo punto de la figura cuando se libera el botn
    * del mouse, al estar dibujando una figura.
    * Se debi realizar esto pues en la implementacin original se asignaban 
    * ambos puntos con el primer click del mouse, pero para guardar los puntos
    * en un archivo, esa implementacin debe ser cambiada
    * @param p2 El punto final de la figura, obtenido al soltar el mouse
    */
   public void setP2(Point2D p2)
   {
      this.punto2 = p2;
   }

   /**
    * Mtodo para poder acceder a un campo provado de la clase Formas
    * @return El primer punto de la figura
    */
   public Point2D getP1()
   {
      return punto1;
   }

   /**
    * Mtodo para poder acceder a un campo provado de la clase Formas
    * @return El segundo punto de la figura
    */
   public Point2D getP2()
   {
      return punto2;
   }

   /**
    * Mtodo abstracto para escribir cada una de las figuras al archivo con
    * extensin ".fig"
    * @param out FileWriter para poder escribir en el mismo archivo.
    * @throws IOException Cuando no puede escribir al archivo o el archivo no 
    * existe.
    */
   public abstract void writeForma(FileWriter out) throws IOException;
   
   /**
    * Mtodo para escribir cada una de las figuras almacenadas como Objeto al
    * archivo con extensin ".eda"
    * @throws IOException Cuando no puede escribir al archivo o el archivo no 
    * existe.
    */
   private void writeObject(ObjectOutputStream out) throws IOException
   {
      // escritura normal
      out.defaultWriteObject();
      // escribe las coordenadas de los 2 puntos
      out.writeDouble(punto1.getX());
      out.writeDouble(punto1.getY());
      out.writeDouble(punto2.getX());
      out.writeDouble(punto2.getY());
   }

   /**
    * Mtodo para leer cada una de las figuras almacenadas como Objeto en el
    * archivo con extensin ".eda"
    * @throws IOException Si ocurre un error de entrada/salida al realizar 
    * la lectura del Stram.
    * @throws ClassNotFoundException Al encontrar un objeto  perteneciente a 
    * una clase no serializable.
    */
   private void readObject(ObjectInputStream in) throws IOException, 
      ClassNotFoundException
   {
      // lectura normal
      in.defaultReadObject();
      // lee las coordenadas del primer punto y define el mismo, luego hace
      // lo mismo con el segundo punto.
      double x = in.readDouble();
      double y = in.readDouble();
      punto1 = new Point2D.Double(x, y);
      x = in.readDouble();
      y = in.readDouble();
      punto2 = new Point2D.Double(x, y);
   }


   private transient Point2D punto1, punto2; // Puntos de las figuras
   // valor siempre constante al escribir en formato Fig3.2 (no usado)
   public static final String FORMAS_SET = "0 1 0 0 100 1 -1  6.40"; 
}


/**
 * Clase utilizada para manejar Rectngulos, y poder guardarlos como objeto
 * (serializar). Extiende la clase Formas
 */
class Rectangulo extends Formas
{
   /**
    * Constructor de la clase, define los puntos que corresponden al inicio
    * y final de la figura.
    * @param punto1 corresponde al punto inicial de la figura
    * @param punto2 corresponde al punto final de la figura
    */
    public Rectangulo (Point2D punto1, Point2D punto2)
   {
      super(punto1, punto2); // En la super clase (Formas) se setean los puntos
   }

   /**
    * Mtodo que se usa para devolver un Rectangle2D para que ste sea dibujado
    * y agregado al arreglo de Shapes, en base a los 2 puntos de la figura
    * @return Un Rectangle2D cuya diagonal est dada por los 2 puntos.
    */
   public Object setForma()
   {
      // obtiene los 2 puntos de la figura y cada una de sus coordenadas
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      double a = p1.getX();
      double b = p1.getY();
      double x = p2.getX();
      double y = p2.getY();
      // crea el Rectangle2D y la retorna
      Rectangle2D rect = new Rectangle2D.Double (a, b, x - a, y - b);
      return rect;
   }

   /**
    * Mtodo usado para escribir al archivo ".fig" un rectngulo segn el 
    * Fig3.2
    * @throws IOException Cuando no puede escribir al archivo o el archivo no 
    * existe.
    */
   public void writeForma(FileWriter out) throws IOException
   {
      // Escribe la identificacin del Rectngulo y valores no usados
      out.write("\n" + RECT_CODE + " " + super.FORMAS_SET + " " + RELLENO
         + "\n");
      // obtiene los 2 puntos de la figura y cada una de sus coordenadas
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      int a = (int)p1.getX();
      int b = (int)p1.getY();
      int x = (int)p2.getX();
      int y = (int)p2.getY();
      // escribe las coordenadas de los puntos al archivo
      out.write("\t" + a + " " + b + " " + a + " " + y + " " + x + " " + y 
         + " " + x + " " + b + " " + a + " " + b);

   }
   private static final String RECT_CODE = new String("2 2");   //polyline box
   // valor siempre constante al escribir en formato Fig3.2 (no usado)
   private static final String RELLENO = new String("0 0 0 0 0 5");
}

/**
 * Clase utilizada para manejar Elpses, y poder guardarlos como objeto
 * (serializar). Extiende la clase Formas
 */
class Elipse extends Formas
{
   /**
    * Constructor de la clase, define los puntos que corresponden al inicio
    * y final de la figura.
    * @param punto1 corresponde al punto inicial de la figura
    * @param punto2 corresponde al punto final de la figura
    */
   public Elipse (Point2D punto1, Point2D punto2)
   {
      super(punto1, punto2); // En la super clase (Formas) se setean los puntos
   }

   /**
    * Mtodo que se usa para devolver una Ellipse2D para que sta sea dibujada
    * y agregada al arreglo de Shapes, en base a los 2 puntos de la figura
    * @return Una Ellipse2D contenida en un Rectangle2D cuya diagonal est dada
    * por los 2 puntos.
    */
   public Object setForma()
   {
      // obtiene los 2 puntos de la figura y cada una de sus coordenadas
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      double a = p1.getX();
      double b = p1.getY();
      double x = p2.getX();
      double y = p2.getY();
      // crea la Ellipse2D y la retorna
      Ellipse2D eli = new Ellipse2D.Double (a, b, x - a, y - b);
      return eli;
   }

   /**
    * Mtodo usado para escribir al archivo ".fig" una Elpse segn el 
    * Fig3.2
    * @throws IOException Cuando no puede escribir al archivo o el archivo no 
    * existe.
    */
   public void writeForma(FileWriter out) throws IOException
   {
      // Escribe la identificacin de la Elpse y valores no usados
      out.write("\n" + ELIP_CODE + " " + super.FORMAS_SET + " " + RELLENO 
         + " ");
      // obtiene los 2 puntos de la figura y cada una de sus coordenadas
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      int x1 = (int)p1.getX();
      int y1 = (int)p1.getY();
      int x2 = (int)p2.getX();
      int y2 = (int)p2.getY();
      // calcula el centro y el radio
      int cx = (x1 + x2) / 2;
      int cy = (y1 + y2) / 2;
      int rx = Math.abs(x1 - x2) / 2;
      int ry = Math.abs(y1 - y2) / 2;
      // escribe las coordenadas de los puntos al archivo
      out.write(cx + " " + cy + " " + rx + " " + ry + " " + x1 + " " + y1 + " "
         + x2 + " " + y2);
   }
   
   //elipse segun diametro
   private static final String ELIP_CODE = new String ("1 2");
   // valor siempre constante al escribir en formato Fig3.2 (no usado)
   private static final String RELLENO = new String ("1 0.0");
}

/**
 * Clase utilizada para manejar Lneas, y poder guardarlos como objeto
 * (serializar). Extiende la clase Formas
 */
class Linea extends Formas
{
   /**
    * Constructor de la clase, define los puntos que corresponden al inicio
    * y final de la figura.
    * @param punto1 corresponde al punto inicial de la figura
    * @param punto2 corresponde al punto final de la figura
    */
   public Linea (Point2D punto1, Point2D punto2)
   {
      super(punto1, punto2); // En la super clase (Formas) se setean los puntos
   }

   /**
    * Mtodo que se usa para devolver una Line2D para que sta sea dibujada
    * y agregada al arreglo de Shapes, en base a los 2 puntos de la figura
    * @return Una Line2D entre los 2 puntos de la figura.
    */
   public Object setForma()
   {
      // obtiene los 2 puntos de la figura.
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      // crea la Line2D y la retorna
      Line2D lin = new Line2D.Double (p1, p2);
      return lin;
   }

   /**
    * Mtodo usado para escribir al archivo ".fig" una Lnea segn el 
    * Fig3.2
    * @throws IOException Cuando no puede escribir al archivo o el archivo no 
    * existe.
    */
   public void writeForma(FileWriter out) throws IOException
   {
      // Escribe la identificacin de la Lnea y valores no usados
      out.write("\n" + LIN_CODE + " " + super.FORMAS_SET + " " + RELLENO 
         + "\n");
      // obtiene los 2 puntos de la figura
      Point2D p1 = this.getP1();
      Point2D p2 = this.getP2();
      // escribe las coordenadas de los puntos al archivo
      out.write("\t" + (int)p1.getX() + " " + (int)p1.getY() + " " 
         + (int)p2.getX() + " " + (int)p2.getY());
   }

   private static final String LIN_CODE = new String ("2 1");  //Polyline linea
   // valor siempre constante al escribir en formato Fig3.2 (no usado)
   private static final String RELLENO = new String ("0 0 12 0 0 2");
}
